﻿hopAddon_Minimap = CreateFrame("Button", "ServerHopMinimap", Minimap);

local Shapes = {
	["ROUND"] = {true, true, true, true},
	["SQUARE"] = {false, false, false, false},
	["CORNER-TOPLEFT"] = {false, false, false, true},
	["CORNER-TOPRIGHT"] = {false, false, true, false},
	["CORNER-BOTTOMLEFT"] = {false, true, false, false},
	["CORNER-BOTTOMRIGHT"] = {true, false, false, false},
	["SIDE-LEFT"] = {false, true, false, true},
	["SIDE-RIGHT"] = {true, false, true, false},
	["SIDE-TOP"] = {false, false, true, true},
	["SIDE-BOTTOM"] = {true, true, false, false},
	["TRICORNER-TOPLEFT"] = {false, true, true, true},
	["TRICORNER-TOPRIGHT"] = {true, false, true, true},
	["TRICORNER-BOTTOMLEFT"] = {true, true, false, true},
	["TRICORNER-BOTTOMRIGHT"] = {true, true, true, false},
}

function hopAddon_Minimap.MinimapUpdate(button)
	local angle = math.rad(button.minimapPos or 225);
	local x, y, q = math.cos(angle), math.sin(angle), 1;
	if x < 0 then 
		q = q + 1;
	end
	if y > 0 then
		q = q + 2;
	end
	local minimapShape = GetMinimapShape and GetMinimapShape() or "ROUND";
	local quadTable = Shapes[minimapShape];
	if quadTable[q] then
		x, y = x*80, y*80;
	else
		local diagRadius = 103.13708498985; --math.sqrt(2*(80)^2)-10
		x = math.max(-80, math.min(x*diagRadius, 80));
		y = math.max(-80, math.min(y*diagRadius, 80));
	end
	button:SetPoint("CENTER", Minimap, "CENTER", x, y);
end

function OnUpdate(self)
	local mx, my = Minimap:GetCenter();
	local px, py = GetCursorPosition();
	local scale = Minimap:GetEffectiveScale();
	px, py = px / scale, py / scale;
	self.minimapPos = math.deg(math.atan2(py - my, px - mx)) % 360;
	self.MinimapUpdate(self);
end

local function wlUpdateMinimapButtonCoord(self)
	local coords = {0, 1, 0, 1};
	local deltaX, deltaY = 0, 0;
	if not self:GetParent().isMouseDown then
		deltaX = (coords[2] - coords[1]) * 0.05;
		deltaY = (coords[4] - coords[3]) * 0.05;
	end
	self:SetTexCoord(coords[1] + deltaX, coords[2] - deltaX, coords[3] + deltaY, coords[4] - deltaY);
end

function OnEnter(self, motion)
	if self.dragging then
		return;
	end
	GameTooltip:SetOwner(self, "ANCHOR_LEFT");
	GameTooltip:SetText("Server Hop", 1, 1, 1);
	GameTooltip:AddLine(HOPADDON_TOGGLE, nil, nil, nil, 1);
	GameTooltip:Show();
end

local function OnLeave(self, motion)
	GameTooltip:Hide();
end

local function OnClick(self, button, down)
	if hopAddon:IsShown() then
		hopAddon:Hide()
	else
		hopAddon:Show()
	end
end

local function OnDragStart(self, button)
	GameTooltip:Hide();
	self:LockHighlight();
	self.dragging = true;
	self.isMouseDown = true;
	self:SetScript("OnUpdate", OnUpdate);
	self.icon:wlUpdateMinimapButtonCoord();
end

local function OnDragStop(self, button)
	self:SetScript("OnUpdate", nil);
	self:UnlockHighlight();
	self.dragging = false;
	self.isMouseDown = false;
	self.icon:wlUpdateMinimapButtonCoord();
end

local function OnMouseDown(self, button)
	self.isMouseDown = true;
	self.icon:wlUpdateMinimapButtonCoord();
end

local function OnMouseUp(self, button)
	self.isMouseDown = false;
	self.icon:wlUpdateMinimapButtonCoord();
end

local button = hopAddon_Minimap
button:SetFrameStrata("MEDIUM");
button:SetWidth(31);
button:SetHeight(31);
button:SetFrameLevel(8);
button:RegisterForClicks("anyUp");
button:RegisterForDrag("LeftButton");
button:SetHighlightTexture([[Interface\Minimap\UI-Minimap-ZoomButton-Highlight]]);
local overlay = button:CreateTexture(nil, "OVERLAY");
overlay:SetWidth(53);
overlay:SetHeight(53);
overlay:SetTexture([[Interface\Minimap\MiniMap-TrackingBorder]]);
overlay:SetPoint("TOPLEFT");
local background = button:CreateTexture(nil, "BACKGROUND");
background:SetWidth(25); background:SetHeight(25);
background:SetTexture([[Interface\Minimap\UI-Minimap-Background]]);
background:SetPoint("TOPLEFT", 2, -4);
local icon = button:CreateTexture(nil, "ARTWORK");
icon:SetWidth(20); icon:SetHeight(20);
icon:SetTexture([[Interface\Icons\Achievement_General_StayClassy]]);
icon:SetPoint("TOPLEFT", 6, -6);
button.icon = icon;
button.isMouseDown = false;

icon.wlUpdateMinimapButtonCoord = wlUpdateMinimapButtonCoord;
icon:wlUpdateMinimapButtonCoord();

button:SetScript("OnEnter", OnEnter);
button:SetScript("OnLeave", OnLeave);
button:SetScript("OnClick", OnClick);
button:SetScript("OnDragStart", OnDragStart);
button:SetScript("OnDragStop", OnDragStop);
button:SetScript("OnMouseDown", OnMouseDown);
button:SetScript("OnMouseUp", OnMouseUp);
